# lib.sh is library of common shell functions for topo-*

PID=$$

# `xnewhost <name>` creates new virtual host.
xnewhost() {
    local X=$1
    # lo
    xunshare $X -- ip link set lo up

    # private /var/run so that e.g. smcrouted can be started
    xnsenter $X -- mount -t tmpfs none /var/run

    # open terminal inside the host for play
    local nterm=${2:-1}
    for i in `seq $nterm`; do
        xnsenter $X -- xfce4-terminal --title=$X$i &
    done
}


# `xunshare <dir> ...`  -- unshare net/mount/... namespaces to be referenced by dir
xunshare() {
    ref=$1
    shift
    echo "create $ref"
    rm -rf "$ref"
    mkdir "$ref"
    touch $ref/mnt $ref/net
    unshare --mount=$ref/mnt --net=$ref/net "$@"
}

# `xnsenter <dir> ...`  -- nsenter net/mount/... namespaces to be referenced by dir
xnsenter() {
    ref=$1
    shift
    nsenter --mount=$ref/mnt --net=$ref/net --wd=. "$@"
}


# `xlink A B` sets up veth links in between nodes A and B going through bridge brAB:
#
#       A  ------- brAB -------  B
#        a-b  brAB.a  brAB.b  b-a
xlink() {
    A=$1
    B=$2
    a="${A,,}" # A -> a
    b="${B,,}" # B -> b
    echo "link $A-$B"

    xnsenter $A -- ip link add $a-$b type veth peer br$A$B.$a
    xnsenter $A -- ip link set br$A$B.$a netns $PID
    xnsenter $A -- ip link set $a-$b up
    ip link set br$A$B.$a up

    xnsenter $B -- ip link add $b-$a type veth peer br$A$B.$b
    xnsenter $B -- ip link set br$A$B.$b netns $PID
    xnsenter $B -- ip link set $b-$a up
    ip link set br$A$B.$b up

    ip link add br$A$B type bridge # vlan_filtering 1 mcast_snooping 0
    ip link set br$A$B.$a master br$A$B
    ip link set br$A$B.$b master br$A$B
    ip link set br$A$B up
}
